@@ -499,13 +499,21 @@ class Deriver(object):
                 continue
 
             # using this fragment and the whole parent molecule, estimate the "missing" FC and size
-            parent = Heritage.get(Heritage.frag_id == user_frag.id).parent
-            if parent is not None:
-                missing_piece_fc = (parent.frag_coeff - user_frag.frag_coeff) - 1.0  # -1.0 because two pieces combine
-                missing_piece_len = len(parent.smile) - len(user_frag.smile)  # approximation
-            else:
-                missing_piece_fc = 3.0  # approximation
-                missing_piece_len = 40  # approximation
+            try:
+                parent = Heritage.get(Heritage.frag_id == user_frag.id).parent
+            # todo: actual exception is deriver.lib_read.FragmentDoesNotExist, check if we can except just this case
+            except Exception as e:  # pylint: disable=broad-except
+                logger.warning(f'Encountered exception {e}')
+                logger.warning('If this exception describes a missing parent in the Heritage table, this bug'
+                               'is known and is being handled as intended.')
+                continue
+
+            # if parent is not None:
+            missing_piece_fc = (parent.frag_coeff - user_frag.frag_coeff) - 1.0  # -1.0 because two pieces combine
+            missing_piece_len = len(parent.smile) - len(user_frag.smile)  # approximation
+            # else:
+            #       missing_piece_fc = 3.0  # approximation
+            #       missing_piece_len = 40  # approximation
 
             # this is what we are going to keep
             seed_frag = (user_frag.smile,
